package com.zskk.dicom.monitor.runner;

import com.zskk.dicom.monitor.config.Configs;
import com.zskk.dicom.monitor.queue.IQueue;
import com.zskk.dicom.monitor.uploader.BaseFileUploader;
import org.apache.http.util.TextUtils;

/**
 * 上传runner基类
 */
public abstract class BaseUploadRunner extends  AbstractUploadRunner {


    BaseUploadRunner(IQueue<String> queue) {
        super(queue);
    }

    @Override
    public boolean upload() {
        Configs.sysLog.info("upload");
        boolean flag = false;
        String filePath = getUploadFilePath();
        if(TextUtils.isEmpty(filePath)) {
            return flag;
        }
        flag = upload(filePath);
        return flag;
    }

    /**
     * 上传文件
     * @param filePath 被删除文件路径
     * @return
     */
    private boolean upload(String filePath) {
        return BaseFileUploader.upload(filePath, getSuccessQueue(), getErrorQueue());
    }

    /**
     * 获取被删除文件路径
     * @return
     */
    protected abstract String getUploadFilePath();

    /**
     * 获取上传成功文件路径应该放入的队列
     * @return
     */
    protected abstract IQueue<String> getSuccessQueue();

    /**
     * 获取上传失败文件应该放入的队列
     * @return
     */
    protected abstract IQueue<String> getErrorQueue();
}