package com.zskk.dicom.monitor.uploader;

import com.zskk.dicom.monitor.config.Configs;
import com.zskk.dicom.monitor.report.ErrReporter;
import com.zskk.dicom.monitor.utils.ExceptionUtil;
import com.zskk.dicom.monitor.utils.FileHashUtil;
import com.zskk.dicom.monitor.utils.MonitorFileUtils;
import org.apache.commons.io.FileUtils;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

public class FileMover implements Runnable {

	File sourceFile;
	File targetFile;

	public FileMover(File sourceFile, File targetFile) {
		this.sourceFile = sourceFile;
		this.targetFile = targetFile;
	}

	@SuppressWarnings("static-access")
	@Override
	public void run() {
		try {
			// 暂停一秒，等待系统处理完文件的占用
			Thread.currentThread().sleep(100);
		} catch (InterruptedException e1) {
			e1.printStackTrace();
		}
		String targetFileStr;
		try {
			targetFileStr = targetFile.getCanonicalPath();
			if (targetFile.exists()) {
				// 如果移动的文件，已经在目标地址存在
				// 那么，比对Hash值，看是否为同一个文件
				// 如果是同一个文件，只需要删除当前文件，不必移除
				// 如果不是同一个文件，另备份目录
				Configs.sysLog.info("备份目标文件已经存在，需要比对文件Hash值");
				String tHash = FileHashUtil.getFileMD5(targetFile);
				String sHash = FileHashUtil.getFileMD5(sourceFile);
				Configs.sysLog.info(sourceFile.getAbsolutePath() + "[" + sHash + "] ----->>----- " + targetFile.getAbsolutePath() + "[" + tHash + "]");
				if (tHash.equals(sHash)) {
					// 2个文件的Hash值相同
					// 不再移动文件，删除原文件
					boolean isSucDel = sourceFile.delete();
					Configs.sysLog.info("删除源文件：" + sourceFile.getAbsolutePath() + "  结果：" + isSucDel);
					if (!isSucDel) {
						// 文件没有成功删除
						ErrReporter.report("文件删除失败:" + sourceFile.getCanonicalPath());
					}
				} else {
					// 2个文件的Hash值不同
					targetFile = new File(targetFileStr + "2");
					if (targetFile.exists()) {
						// 如果备份文件也存在，
						// 先删除，再将源文件移至此文件
						targetFile.delete();
					}
					// 移动文件到另一个名称下
					try {
						FileUtils.moveFile(sourceFile, targetFile);
						Configs.sysLog.info("文件移动---------:" + sourceFile.getCanonicalPath() + " to:" + targetFileStr + "2    成功");
					} catch (IOException ioe) {
						if (ioe instanceof FileNotFoundException) {
							// 源文件已不存在
							Configs.sysLog.info("文件移动-------:" + sourceFile.getCanonicalPath() + " to:" + targetFileStr + "2    " + ioe.getMessage());
						} else {
							// 文件没有成功删除
							ErrReporter.report("文件移动失败---------:" + sourceFile.getCanonicalPath() + " to:" + targetFileStr + "2    " + ioe.getMessage());
						}
					}
				}
			} else {
				// 上传完成后，移除文件
				try {
					FileUtils.moveFile(sourceFile, targetFile);
					Configs.sysLog.info("文件移动-----------:" + sourceFile.getCanonicalPath() + " to:" + targetFile.getAbsolutePath() + "    成功");
				} catch (IOException ioe) {
					if (ioe instanceof FileNotFoundException) {
						// 源文件已不存在
						Configs.sysLog.info("文件移动------------:" + sourceFile.getCanonicalPath() + " to:" + targetFile.getAbsolutePath() + "    " + ioe.getMessage());
					} else {
						// 文件没有成功删除
						ErrReporter.report("文件移动失败----------:" + sourceFile.getCanonicalPath() + " to:" + targetFileStr + "    " + ioe.getMessage());
					}
				}
			}


		} catch (IOException e) {
			e.printStackTrace();
			// 文件没有成功删除
			ErrReporter.report("文件(" + targetFile.getAbsolutePath() + ")获取getCanonicalPath时遇到错误:" + ExceptionUtil.getExceptionTxt(e));
		}
		// 移除空的目录
		MonitorFileUtils.removeBeforeTodayEmptyDir(sourceFile.getParentFile());

	}

}
